@@ -155,6 +155,7 @@ enum lttng_event_context_type {
 	LTTNG_EVENT_CONTEXT_VGID		= 38,
 	LTTNG_EVENT_CONTEXT_VEGID		= 39,
 	LTTNG_EVENT_CONTEXT_VSGID		= 40,
+	LTTNG_EVENT_CONTEXT_TIME_NS		= 41,
 };
 
 enum lttng_event_field_type {
@@ -515,7 +516,7 @@ extern int lttng_disable_event(struct lttng_handle *handle,
  *
  * Return 0 on success else a negative LTTng error code.
  */
-int lttng_disable_event_ext(struct lttng_handle *handle,
+extern int lttng_disable_event_ext(struct lttng_handle *handle,
 		struct lttng_event *ev, const char *channel_name,
 		const char *filter_expression);
 
