@@ -1,19 +1,9 @@
 /*
- * Copyright (C) 2014 - Jérémie Galarneau <jeremie.galarneau@efficios.com>
- *                      David Goulet <dgoulet@efficios.com>
+ * Copyright (C) 2014 Jérémie Galarneau <jeremie.galarneau@efficios.com>
+ * Copyright (C) 2014 David Goulet <dgoulet@efficios.com>
  *
- * This library is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License, version 2.1 only,
- * as published by the Free Software Foundation.
+ * SPDX-License-Identifier: LGPL-2.1-only
  *
- * This library is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
- * for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this library; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 
 #ifndef LTTNG_LOAD_H
@@ -32,12 +22,12 @@ struct lttng_load_session_attr;
 /*
  * Return a newly allocated load session attribute object or NULL on error.
  */
-struct lttng_load_session_attr *lttng_load_session_attr_create(void);
+extern struct lttng_load_session_attr *lttng_load_session_attr_create(void);
 
 /*
  * Free a given load session attribute object.
  */
-void lttng_load_session_attr_destroy(struct lttng_load_session_attr *attr);
+extern void lttng_load_session_attr_destroy(struct lttng_load_session_attr *attr);
 
 
 /*
@@ -45,14 +35,14 @@ void lttng_load_session_attr_destroy(struct lttng_load_session_attr *attr);
  */
 
 /* Return session name. NULL indicates all sessions must be loaded. */
-const char *lttng_load_session_attr_get_session_name(
+extern const char *lttng_load_session_attr_get_session_name(
 	struct lttng_load_session_attr *attr);
 /*
  * Return input URL. A NULL value indicates the default session
  * configuration location. The URL format used is documented in lttng-create(1).
  * NULL indicates that the default session configuration path is used.
  */
-const char *lttng_load_session_attr_get_input_url(
+extern const char *lttng_load_session_attr_get_input_url(
 	struct lttng_load_session_attr *attr);
 
 /*
@@ -61,7 +51,7 @@ const char *lttng_load_session_attr_get_input_url(
  * same name already exists. If such a session exists, it is destroyed before
  * the replacement is loaded.
  */
-int lttng_load_session_attr_get_overwrite(
+extern int lttng_load_session_attr_get_overwrite(
 	struct lttng_load_session_attr *attr);
 
 /*
@@ -71,7 +61,7 @@ int lttng_load_session_attr_get_overwrite(
  *
  * NULL indicates no override will be applied on configuration load.
  */
-const char *lttng_load_session_attr_get_override_url(
+extern const char *lttng_load_session_attr_get_override_url(
 	struct lttng_load_session_attr *attr);
 
 /*
@@ -81,7 +71,7 @@ const char *lttng_load_session_attr_get_override_url(
  *
  * NULL indicates no control URL override will be applied on configuration load.
  */
-const char *lttng_load_session_attr_get_override_ctrl_url(
+extern const char *lttng_load_session_attr_get_override_ctrl_url(
 	struct lttng_load_session_attr *attr);
 
 /*
@@ -91,7 +81,7 @@ const char *lttng_load_session_attr_get_override_ctrl_url(
  *
  * NULL indicates no data URL override will be applied on configuration load.
  */
-const char *lttng_load_session_attr_get_override_data_url(
+extern const char *lttng_load_session_attr_get_override_data_url(
 	struct lttng_load_session_attr *attr);
 
 /*
@@ -102,7 +92,7 @@ const char *lttng_load_session_attr_get_override_data_url(
  * NULL indicates no session name override will be applied on configuration
  * load.
  */
-const char *lttng_load_session_attr_get_override_session_name(
+extern const char *lttng_load_session_attr_get_override_session_name(
 	struct lttng_load_session_attr *attr);
 
 /*
@@ -116,7 +106,7 @@ const char *lttng_load_session_attr_get_override_session_name(
  * Set the name of the session to load. A NULL name means all sessions
  * found at the input URL will be loaded.
  */
-int lttng_load_session_attr_set_session_name(
+extern int lttng_load_session_attr_set_session_name(
 	struct lttng_load_session_attr *attr, const char *session_name);
 
 /*
@@ -125,7 +115,7 @@ int lttng_load_session_attr_set_session_name(
  *
  * Note that file:// is the only supported URL format.
  */
-int lttng_load_session_attr_set_input_url(
+extern int lttng_load_session_attr_set_input_url(
 	struct lttng_load_session_attr *attr, const char *url);
 
 /*
@@ -133,7 +123,7 @@ int lttng_load_session_attr_set_input_url(
  * loaded sessions will be destroyed and be replaced by the session(s) being
  * loaded.
  */
-int lttng_load_session_attr_set_overwrite(
+extern int lttng_load_session_attr_set_overwrite(
 	struct lttng_load_session_attr *attr, int overwrite);
 
 /*
@@ -153,7 +143,7 @@ int lttng_load_session_attr_set_overwrite(
  *
  * See lttng-create(1) for more detail.
  */
-int lttng_load_session_attr_set_override_url(
+extern int lttng_load_session_attr_set_override_url(
 	struct lttng_load_session_attr *attr, const char *url);
 
 /*
@@ -166,7 +156,7 @@ int lttng_load_session_attr_set_override_url(
  *
  * See lttng-create(1) for more detail.
  */
-int lttng_load_session_attr_set_override_ctrl_url(
+extern int lttng_load_session_attr_set_override_ctrl_url(
 	struct lttng_load_session_attr *attr, const char *url);
 
 /*
@@ -179,7 +169,7 @@ int lttng_load_session_attr_set_override_ctrl_url(
  *
  * See lttng-create(1) for more detail.
  */
-int lttng_load_session_attr_set_override_data_url(
+extern int lttng_load_session_attr_set_override_data_url(
 	struct lttng_load_session_attr *attr, const char *url);
 
 /*
@@ -188,7 +178,7 @@ int lttng_load_session_attr_set_override_data_url(
  * Loading a configuration file defining multiple sessions will fail if a
  * session name is provided.
  */
-int lttng_load_session_attr_set_override_session_name(
+extern int lttng_load_session_attr_set_override_session_name(
 	struct lttng_load_session_attr *attr, const char *session_name);
 
 /*
@@ -199,7 +189,7 @@ int lttng_load_session_attr_set_override_session_name(
  *
  * Returns 0 on success or a negative LTTNG_ERR value on error.
  */
-int lttng_load_session(struct lttng_load_session_attr *attr);
+extern int lttng_load_session(struct lttng_load_session_attr *attr);
 
 #ifdef __cplusplus
 }
