@@ -21,7 +21,7 @@
 #include <common/utils.h>
 
 #include "../command.h"
-#include <lttng/rotation.h>
+#include <lttng/lttng.h>
 
 static char *opt_session_name;
 static struct mi_writer *writer;
