@@ -1,18 +1,8 @@
 /*
- * Copyright (C) 2013 - David Goulet <dgoulet@efficios.com>
+ * Copyright (C) 2013 David Goulet <dgoulet@efficios.com>
  *
- * This program is free software; you can redistribute it and/or modify it
- * under the terms of the GNU General Public License, version 2 only, as
- * published by the Free Software Foundation.
+ * SPDX-License-Identifier: GPL-2.0-only
  *
- * This program is distributed in the hope that it will be useful, but WITHOUT
- * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
- * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
- * more details.
- *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc., 51
- * Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #define _LGPL_SOURCE
@@ -29,7 +19,7 @@
 
 #include <common/utils.h>
 #include <common/mi-lttng.h>
-#include <lttng/snapshot.h>
+#include <lttng/lttng.h>
 
 #include "../command.h"
 
@@ -634,10 +624,11 @@ int cmd_snapshot(int argc, const char **argv)
 		case OPT_MAX_SIZE:
 		{
 			uint64_t val;
-			const char *opt = poptGetOptArg(pc);
+			const char *max_size_arg = poptGetOptArg(pc);
 
-			if (utils_parse_size_suffix((char *) opt, &val) < 0) {
-				ERR("Unable to handle max-size value %s", opt);
+			if (utils_parse_size_suffix((char *) max_size_arg, &val) < 0) {
+				ERR("Unable to handle max-size value %s",
+						max_size_arg);
 				cmd_ret = CMD_ERROR;
 				goto end;
 			}
