@@ -280,7 +280,7 @@ void test_epoll(void)
 	ret = epoll_ctl(epollfd, EPOLL_CTL_ADD, wait_fd, &epoll_event);
 	if (ret < 0) {
 		perror("[epoll] add");
-		goto end;
+		goto error;
 	}
 
 	if (timeout > 0) {
@@ -301,6 +301,11 @@ void test_epoll(void)
 		perror("epoll_wait");
 	}
 
+error:
+	ret = close(epollfd);
+	if (ret) {
+		perror("close");
+	}
 end:
 	return;
 }
@@ -323,7 +328,7 @@ void test_pepoll(void)
 	ret = epoll_ctl(epollfd, EPOLL_CTL_ADD, wait_fd, &epoll_event);
 	if (ret < 0) {
 		perror("[eppoll] add");
-		goto end;
+		goto error;
 	}
 
 	if (timeout > 0) {
@@ -344,6 +349,11 @@ void test_pepoll(void)
 		perror("epoll_pwait");
 	}
 
+error:
+	ret = close(epollfd);
+	if (ret) {
+		perror("close");
+	}
 end:
 	return;
 }
@@ -556,7 +566,7 @@ void epoll_pwait_invalid_pointer(void)
 	ret = epoll_ctl(epollfd, EPOLL_CTL_ADD, wait_fd, &epoll_event);
 	if (ret < 0) {
 		perror("[eppoll] add");
-		goto end;
+		goto error;
 	}
 
 	ret = syscall(SYS_epoll_pwait, epollfd,
@@ -574,6 +584,11 @@ void epoll_pwait_invalid_pointer(void)
 		perror("# epoll_pwait");
 	}
 
+error:
+	ret = close(epollfd);
+	if (ret) {
+		perror("close");
+	}
 end:
 	return;
 }
@@ -600,7 +615,7 @@ void epoll_pwait_int_max(void)
 	ret = epoll_ctl(epollfd, EPOLL_CTL_ADD, wait_fd, &epoll_event);
 	if (ret < 0) {
 		perror("[eppoll] add");
-		goto end;
+		goto error;
 	}
 
 	ret = syscall(SYS_epoll_pwait, epollfd, &epoll_event, INT_MAX, -1,
@@ -618,6 +633,11 @@ void epoll_pwait_int_max(void)
 		perror("# epoll_pwait");
 	}
 
+error:
+	ret = close(epollfd);
+	if (ret) {
+		perror("close");
+	}
 end:
 	return;
 }
@@ -774,7 +794,7 @@ void epoll_pwait_concurrent_munmap(void)
 			-1, 0);
 	if (epoll_event == MAP_FAILED) {
 		perror("mmap");
-		goto end;
+		goto error;
 	}
 
 	for (i = 0; i < MAX_FDS; i++) {
@@ -787,7 +807,7 @@ void epoll_pwait_concurrent_munmap(void)
 		ret = epoll_ctl(epollfd, EPOLL_CTL_ADD, fds[i], epoll_event);
 		if (ret < 0) {
 			perror("[eppoll] add");
-			goto end_unmap;
+			goto error_unmap;
 		}
 	}
 	stop_thread = 0;
@@ -795,7 +815,7 @@ void epoll_pwait_concurrent_munmap(void)
 			(void *) epoll_event);
 	if (ret != 0) {
 		fprintf(stderr, "[error] pthread_create\n");
-		goto end_unmap;
+		goto error_unmap;
 	}
 
 	ret = epoll_pwait(epollfd, epoll_event, 1, 1, NULL);
@@ -816,9 +836,9 @@ void epoll_pwait_concurrent_munmap(void)
 	ret = pthread_join(writer, NULL);
 	if (ret) {
 		fprintf(stderr, "[error] pthread_join\n");
-		goto end_unmap;
+		goto error_unmap;
 	}
-end_unmap:
+error_unmap:
 	for (i = 0; i < MAX_FDS; i++) {
 		ret = close(fds[i]);
 		if (ret != 0) {
@@ -831,6 +851,11 @@ end_unmap:
 		perror("munmap");
 	}
 
+error:
+	ret = close(epollfd);
+	if (ret) {
+		perror("close");
+	}
 end:
 	return;
 }
